#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include <GL/glew.h>
#ifdef __APPLE__
#include <GLUT/glut.h>
#else

#ifdef __unix__
#include <GL/glxew.h>
#else
#include <GL/wglew.h>
#endif

#include <GL/glut.h>
#endif

void idle(void);
void display(void);
void reshape(int x, int y);
void keyb(unsigned char key, int x, int y);
void sleep_ms(unsigned int ms);

int vsync = -1;
Display *dpy;
Window xwin;

int main(int argc, char **argv)
{
	if(argv[1]) {
		if(strcmp(argv[1], "-vsync") == 0) {
			vsync = 1;
		}
		if(strcmp(argv[1], "-novsync") == 0) {
			vsync = 0;
		}
	}

	glutInit(&argc, argv);
	glutInitWindowSize(1024, 768);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	glutCreateWindow("foo");

	glutIdleFunc(idle);
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyb);

	glewInit();

	if(vsync != -1) {
#ifdef __unix__
		if(GLXEW_EXT_swap_control) {
			Display *dpy = glXGetCurrentDisplay();
			Window xwin = glXGetCurrentDrawable();
			printf("using GLX_EXT_swap_control\n");
			glXSwapIntervalEXT(dpy, xwin, vsync);
		} else if(GLXEW_MESA_swap_control) {
			printf("using GLX_MESA_swap_control\n");
			glXSwapIntervalMESA(vsync);
		} else if(GLXEW_SGI_swap_control) {
			printf("using GLX_SGI_swap_control\n");
			glXSwapIntervalSGI(vsync);
		}
#endif
#ifdef WIN32
		if(WGLEW_EXT_swap_control) {
			wglSwapIntervalEXT(vsync);
		}
#endif
	}

	glutMainLoop();
	return 0;
}

void idle(void)
{
	glutPostRedisplay();
}

void display(void)
{
	static int x;
	static const float color[][3] = {
		{0.2, 0.4, 1.0},
		{1.0, 0.4, 0.2}
	};

	glClear(GL_COLOR_BUFFER_BIT);

	glColor3fv(color[x]);
	x = (x + 1) & 1;

	glBegin(GL_QUADS);
	glVertex2f(-1, -1);
	glVertex2f(1, -1);
	glVertex2f(1, 1);
	glVertex2f(-1, 1);
	glEnd();

	glutSwapBuffers();
	assert(glGetError() == GL_NO_ERROR);

	sleep_ms(2);
}

void reshape(int x, int y)
{
	glViewport(0, 0, x, y);
}

void keyb(unsigned char key, int x, int y)
{
	if(key == 27) {
		exit(0);
	}
}

#if defined(__unix__) || defined(__APPLE__)
#include <unistd.h>
void sleep_ms(unsigned int ms)
{
	usleep(ms * 1000);
}
#else
#include <windows.h>
void sleep_ms(unsigned int ms)
{
	Sleep(ms);
}
#endif
